/**
     errors.c

     error handling

     Petr Zika 2002
*/

#define _XOPEN_SOURCE 500

#include <stdlib.h>
#include <stdarg.h>
#include <setjmp.h>
#include <errno.h>
#include "../comdefs.h"
#include "errors.h"
#include "../utils/utils.h"
#include "../io/io.h"

extern int vsnprintf(char *str, size_t size, const char *format, va_list ap);
     
GLOBAL jmp_buf handler_address;
GLOBAL int handler_set = 0;
GLOBAL int handler_active = 0;

 
extern const char *sys_errlist[];
extern int sys_nerr;
                                
/* print error string */
GLOBAL void perror(char *str)
{
   char *s;

   if (errno > sys_nerr - 1)
      error("%s : unknown error");
   else { 
      s = (char *)sys_errlist[errno];
      if (s) 
         error("%s: %s",str,s);
      else 
         error("%s : unknown error");
   }
}

/** standard error message */
GLOBAL void error(char *str, ...)
{
   int len, n;
	va_list v;
	char *s;
	
   clear_handler();  /* do not use handler */

   len = 64;

	s = MALLOC(len);
	
	/* try to find the right string length */
	while (1) {
	   va_start(v,str);
      n = vsnprintf(s,len,str,v);
      va_end(v);

	   if (n > -1 && n < len) 
		   break;    /* content fits into string */
   
		if (n > -1) 
			len = n + 1;  /* exactly */
		else
			len *= 2;     /* guess */

		free(s);
		s = MALLOC(len);
   }
	
	fprintf(stderr,HELL_NAME ": %s\n",s);
	free(s);

   reset_handler(); /* use handler again */
}
/** fatal error, usually memory allocation */
GLOBAL void NORETURN fatal(char *str)
{
	fprintf(stderr,HELL_NAME ": fatal: %s\n",str);
	exit(1); /* regularly quit */
}

/** unrecoverable error, probably i/o */
GLOBAL void NORETURN die(void)
{
   exit(2); /* return different code to distinguish */
}

#ifdef TEST_ERRORS

int f(int x)
{
    if (x <= 0) return_handler();
    return x % 3;
}

int main(void)
{
    int i; 
  	
    if (set_handler()) {
       error("error test #%d%c",1,'.');
		 clear_handler();
    } 

    printf("program is running\n");

    for (i = 10; i > -10; i--) 
        printf("f(%d) = %d\n",i,f(i));

    printf("program continues\n");
    
    fatal("end now!");

    return 0;
}

#endif

