/**
     errors.h

     error handling

     Petr Zika 2002
*/

#ifndef _ERRORS_H
#define _ERRORS_H

#include <setjmp.h>
#include "../comdefs.h"

/* long jump handler maintenance */

EXPORT jmp_buf handler_address;
EXPORT int handler_set;
EXPORT int handler_active;

/* do not set anything when in use, else set & activate, return setjmp's return value */
#define create_handler() (handler_set ? -1 : (handler_set = handler_active = 1, setjmp(handler_address)))
/* remove handler from use */
#define destroy_handler() do { handler_set = handler_active = 0; } while (0)
/* prevent handler from working */
#define clear_handler() do { if (handler_set) handler_active = 0; } while (0)
/* set up handler again */
#define reset_handler() do { if (handler_set) handler_active = 1; } while (0)
/* return to handler creator, signalling  */
#define return_handler(ret_val) do { if (handler_active) longjmp(handler_address,ret_val); } while (0)

/**EXPORT void perror(char *str);*/

EXPORT void perror(char *str);
EXPORT void error(char *str, ...);
EXPORT void NORETURN fatal(char *str);
EXPORT void NORETURN die(void);

#endif /* ifndef _ERRORS_H */

