/**
     expand.c
 
     wildcard expansion
 
     Petr Zika 2002
*/               

#include <string.h>
#include <glob.h>
#include "../comdefs.h"
#include "expand.h"

LOCAL glob_t globbuf;         /* structure for glob() */
LOCAL int globfull = 0;       /* globbuf is allocated flag */

/** expand word with wildcards */
GLOBAL char **expand(char *str)
{
	/* expand, do not sort, ignore chdir errors, append slashes to dirs 
	   do not return original pattern in case nothing matches */
	if (glob(str,GLOB_MARK|GLOB_NOSORT,NULL,&globbuf))
      return NULL;

   return globbuf.gl_pathv;
}

/** free structures used for wildcard expansion */
GLOBAL void free_expand(void)
{
	if (globfull) { 
		globfree(&globbuf);
		globfull = 0;
	}
}

#ifdef TEST_EXPAND

#include <stdio.h>

#define MAXS 256

int main(int argc, char **argv)
{
   int i, j, k;
	char **arr;
   char pat[MAXS + 1];
	char c;

	for (i = 1, j = 0; i < argc; i++)
	{
      if (((c = argv[i][0]) == '*' || c == '?') && argv[i][1] == '\0')
         pat[j++] = c;
		else {

         for (k = 0; argv[i][k]; k++) {
             c = argv[i][k];
   			if (c == '\\' || c == '[' || c == '*' || c == '?') {
	   		   pat[j++] = '\\';
		   		pat[j++] = c;
			   } else 
				  pat[j++] = c;
         }
		}
	}

   pat[j] = '\0';
	printf("expanding %s:\n",pat);

	arr = expand(pat);
	
	if (!arr)
		printf("error expanding!\n");

	j = 0;
	while (arr[j] != NULL) {
		printf("%s\n",arr[j]);
		j++;
	}
	putchar('\n');

	(void)free_expand();

   return 0;
}

#endif

