/**
       main.c
 
       main() routine
 
       Petr Zika 2002
*/
                 
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include "../comdefs.h"
#include "main.h"
#include "../env/env.h"
#include "../io/io.h"
#include "../lexer/lexer.h"
#include "../grammar/grcheck.h"
#include "../grammar/grbuild.h"
#include "../errors/errors.h"

/* inherited environment */
extern char **environ;

/* whether some process is currently running */
GLOBAL int is_running;

/** interrupt signal handler */
void signal_handler(int signal)
{
   if (!is_running) {
		fprintf(stderr,"\nuse EOF to leave hell\n");
   }
	/* ignore ^C in shell*/
}


/** main routine */
int main(int argc, char **argv)
{
	struct sigaction act;
	int script = 0;
	char *s;
	int err, err_no;
	char cmd_line[MAX_LINE + 1];

	err = 0;

   if (argc > 2) {
		error("usage: %s [script]",argv[0]);
		return 1;
   }

	if (argc == 2) {
		close(stdin);
		if (open(argv[1],O_RDONLY) == -1) {
			perror(argv[1]);
			return 1;
		}
		script = 1;
	}
	
	fprintf(stderr,"\n" HELL_NAME " v" HELL_VERSION " by Petr Zika 2002\n");
	
   is_running = 0;
   sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	act.sa_handler = signal_handler;

   if (sigaction(SIGINT,&act,NULL) == -1) {
		perror("signal handler");
		error("init error");
		return 1;
	}

   if (init_vars(environ)) {
		error("init error");
		return 1;

	}

   while (1) {
loop_start:
      is_running = 0;

		if (!script) {
			if ((s = get_variable("PWD")) == NULL) {
				error("error reading work directory");
				err = 1;
				goto clean_up;
			}
			printf(HELL_NAME ":%s> ",s);
		}

      errno = 0; /* reason is ambiguous NULL of fgets */

		if (fgets(cmd_line,MAX_LINE,stdin) == NULL) {
			err_no = errno;
			if (err_no != 0) {
				/* interrupt could be ^C */
			   if (err_no == EINTR) goto loop_start;
			   perror("fgets");
			   err = 1;
			   goto clean_up;
			} else 
				/* EOF means exit */
				goto clean_up;
		}

		/* accept input only till newline */
		s = strchr(cmd_line,'\n');
		if (s) *s = '\0';
		
      if (grammar_check(cmd_line)) {
		   /* if it doesn't seem okay, don't care at all */
			error("ignoring input line");
			goto loop_start;
		}
		
      /* there are tons of error messages inside, so don't write any more */
	   if (grammar_build(cmd_line) == -2) break;  /* leave shell */

	}

clean_up:
	
   putchar('\n');
	done_vars();
	
	return err;
}

#ifdef TEST_MAIN

int main(int argc, char **argv)
{
   char **envp = environ;

   /* TODO add global variables: PWD ... */


   while (*argv)
      printf("%s ",*argv++);
   putchar('\n');

   while (*envp)
      printf("%s\n",*envp++);

   return 0;
}

#endif

