/**
       tree.h
 
       parsed commands storage structures
 
       Petr Zika 2002
*/
                 
#ifndef _TREE_H
#define _TREE_H

#include "../comdefs.h"

/* redirection types */
#define RED_NONE 0
#define RED_IN   1
#define RED_OUT  2
#define RED_2OUT 3
#define RED_APP  4
#define RED_2APP 5

/* separator types */
#define SEP_OR   6
#define SEP_AND  7

typedef struct redir {
	int type;
   char *name;
} redir;

typedef struct redir_list {
   struct redir_list *next;
   struct redir_list *last;
   int type;
   char *name;
} redir_list;

typedef struct arg_list {
   struct arg_list *next;
   struct arg_list *last;
   char *name;
} arg_list;

typedef struct cmd_list {
   struct cmd_list *next;
   struct cmd_list *last;
	char *name;
	arg_list *args;
	redir_list *redirs;
   redir in;
   redir out;
   redir err;
} cmd_list;

EXPORT void print_arg_list(arg_list *a);
EXPORT void print_redir_list(redir_list *r);
EXPORT void print_cmd_list(cmd_list *r);

EXPORT void free_arg_list(arg_list *a);
EXPORT void free_redir_list(redir_list *r);
EXPORT void free_cmd_list(cmd_list *c);

EXPORT void prune_redir_list(cmd_list *c, int pipe_out);
EXPORT void prune_cmd_list(cmd_list *c);

EXPORT char **create_argv(arg_list *a);
EXPORT void free_argv(char **a);

#endif /* ifndef _TREE_H */

